/////////////////////////////////////////////////////////////////////////////
// Name:        OSCWindow.h
// Purpose:     
// Author:      jrg piringer
// Modified by: 
// Created:     04/05/2006 23:28:28
// RCS-ID:      
// Copyright:   
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef _OSCWINDOW_H_
#define _OSCWINDOW_H_

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma interface "OSCWindow.h"
#endif

/*!
 * Includes
 */

#include "osc/OscOutboundPacketStream.h"
#include "ip/UdpSocket.h"

#include "defsext.h"
#include "edit.h" 

#include "settings.h"
#include "PreferencesDialog.h"
#include "XYPadControl.h"
#include "capture.h"

/*!
 * Forward declarations
 */


/*!
 * Control identifiers
 */

////@begin control identifiers
#define wxID_TESTMENU 10001
#define ID_OSCWINDOW 10000
#define ID_MSGDISPLAY 14000
#define ID_CHUCK_FILE 14001
#define ID_CHUCK_EXEC 14002
#define ID_CLOSE_CONTROL 14003
#define ID_OUTPUT_LIST 14004
#define ID_CHUCK_VM 14005
#define ID_KILL_VM 14006
#define ID_SHRED_LIST 14007
#define ID_VM_REMOVE 14008
#define ID_VM_REPLACE 14009

#define ID_CTRLPAGE_ROOT 11000

// OSC control IDs
#define ID_CONTROLROOT 15000
#define ID_BUTTON1 15000
#define ID_SLIDER1 15100
#define ID_TOGGLE1 15200
#define ID_XYPAD1  15300

/*!
 * Compatibility
 */

#ifndef wxCLOSE_BOX
#define wxCLOSE_BOX 0x1000
#endif

/*!
 * MyDialog class declaration
 */

enum ctrl_type {OSCBUTTON, OSCSLIDER, OSCTOGGLE, OSCXYPAD};

class OSCPipedProcess;
class OSCWindow: public wxFrame
{    
    DECLARE_DYNAMIC_CLASS( OSCWindow )
    DECLARE_EVENT_TABLE()

	wxStaticText *msglabel;
	//wxStaticText *chuckFileName;
	wxString chuckFileName;

	wxNotebook *notebook;
	wxFlexGridSizer *topsizer;

	Edit *m_edit;

	PreferencesDialog *prefDialog;
	
	wxListCtrl *shredList;
	wxToggleButton *vmToggle;
	wxButton *removeShred;
	wxButton *replaceShred;
	
	wxListBox *lbox;
	OSCProcessesArray m_running;
	wxTimer m_timerIdleWakeUp;
	
	bool vmStarted;
	long vmId;
	bool quitAfterVM;
	
	wxArrayString includes;

public:
    /// Constructors
    OSCWindow();
    //~OSCWindow();

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );

    /// Should we show tooltips?
    static bool ShowToolTips();
    
    void setChuckFileName(wxString fn);
    wxString getChuckFileName() {return chuckFileName;}

	void OnQuit(wxCommandEvent& event);
	void OnAbout(wxCommandEvent& event);
	void OnPreferences(wxCommandEvent& event);

    void OnClose (wxCloseEvent &event);
    bool Close(bool force = false);
    
    //! file
    void OnFileNew (wxCommandEvent &event);
    void OnFileNewFrame (wxCommandEvent &event);
    void OnFileOpen (wxCommandEvent &event);
    void OnFileOpenFrame (wxCommandEvent &event);
    void OnFileSave (wxCommandEvent &event);
    void OnFileSaveAs (wxCommandEvent &event);
    void OnFileClose (wxCommandEvent &event);
    //! properties
    void OnProperties (wxCommandEvent &event);
    //! print
    void OnPrintSetup (wxCommandEvent &event);
    void OnPrintPreview (wxCommandEvent &event);
    void OnPrint (wxCommandEvent &event);
	//! edit events
    void OnEdit (wxCommandEvent &event);

	void chuckCommand(wxString command, int flags = wxEXEC_ASYNC|wxEXEC_NOHIDE, wxNotebookPage *ctrltab = NULL);
	void OnExecChuck(wxCommandEvent& event);
	void OnChuckVM(wxCommandEvent& event);
	void OnKillVM(wxCommandEvent& event);
	void OnVMRemove(wxCommandEvent& event);
	void OnVMReplace(wxCommandEvent& event);
	
	void OnShredSelected(wxListEvent &event);
	void OnShredDeselected(wxListEvent &event);
	
    void OnCloseControl(wxCommandEvent& event);
    
    void OnSelectOutputList(wxCommandEvent& event);

	void OnSlider(wxCommandEvent& event);
	void OnButton(wxCommandEvent& event);
	void OnToggle(wxCommandEvent& event);
	void OnXYPad(XYPadEvent& event);

	// helper functions
	bool parseFileContent(wxString fileContent, wxPanel *panel);
	void createControllerTab(wxString fname, wxString fileContent);
	wxNotebookPage *createControllerTabs(wxString fname, wxString fileContent);
	void createEditTab();
	void createControls();
	void createControl(wxPanel* panel, wxFlexGridSizer *gridSizer, ctrl_type type, wxString name, int nr);
	void sendValue(int ctrlnum, float val1, float val2 = 0.0);

	void FileOpen(wxString fname);
	wxRect DeterminePrintSize();
	
	void OnIdle(wxIdleEvent& event);
	void OnProcessTerminated(OSCPipedProcess *process, wxNotebookPage *nbpage);
	//wxListBox *GetLogListBox() const {return lbox;}
	void AppendLog(wxString str);
    void AddAsyncProcess(OSCPipedProcess *process)
    {
        if (m_running.IsEmpty())
        {
            // we want to start getting the timer events to ensure that a
            // steady stream of idle events comes in -- otherwise we
            // wouldn't be able to poll the child process input
            m_timerIdleWakeUp.Start(100);
        }
        //else: the timer is already running

        m_running.Add(process);
    }
	void RemoveAsyncProcess(OSCPipedProcess *process)
    {
        m_running.Remove(process);

        if (m_running.IsEmpty())
        {
            // we don't need to get idle events all the time any more
            m_timerIdleWakeUp.Stop();
        }
    }
    
	SETTING_STRING(osc_adress);
	SETTING_LONG(osc_port);
	SETTING_STRING(chuck_path);
	SETTING_STRING(last_chuck_file);
	SETTING_BOOL(capture_output);
};

#endif
    // _OSCWINDOW_H_
